@@ -1,5 +1,5 @@
 // systemtap cache manager
-// Copyright (C) 2006-2009 Red Hat Inc.
+// Copyright (C) 2006-2019 Red Hat Inc.
 //
 // This file is part of systemtap, and is free software.  You can
 // redistribute it and/or modify it under the terms of the GNU General
@@ -294,16 +294,16 @@ clean_cache(systemtap_session& s)
         {
           //interval not passed, don't continue
           if (s.verbose > 1)
-            clog << _F("Cache cleaning skipped, interval not reached %lu s / %lu s.",
-                       (current_time.tv_sec-sb.st_mtime), cache_clean_interval)  << endl;
+            clog << _F("Cache cleaning skipped, interval not reached %ld s / %lu s.",
+                       (long) (current_time.tv_sec-sb.st_mtime), cache_clean_interval)  << endl;
           return;
         }
       else
         {
           //interval reached, continue
           if (s.verbose > 1)
-            clog << _F("Cleaning cache, interval reached %lu s > %lu s.",
-                       (current_time.tv_sec-sb.st_mtime), cache_clean_interval)  << endl;
+            clog << _F("Cleaning cache, interval reached %ld s > %lu s.",
+                       (long) (current_time.tv_sec-sb.st_mtime), cache_clean_interval)  << endl;
         }
 
       // glob for all files that look like hashes
