@@ -1,7 +1,7 @@
 /*
   Common functions used by the NSS-aware code in systemtap.
 
-  Copyright (C) 2009-2014 Red Hat Inc.
+  Copyright (C) 2009-2018 Red Hat Inc.
 
   This file is part of systemtap, and is free software.  You can
   redistribute it and/or modify it under the terms of the GNU General Public
@@ -33,7 +33,6 @@ extern "C" {
 #include <glob.h>
 #include <sys/stat.h>
 #include <sys/utsname.h>
-
 #include <nss.h>
 #include <nspr.h>
 #include <ssl.h>
@@ -44,6 +43,12 @@ extern "C" {
 #include <keyhi.h>
 #include <secder.h>
 #include <cert.h>
+#ifdef HAVE_HTTP_SUPPORT
+#include <openssl/bio.h>
+#include <openssl/x509.h>
+#include <openssl/pem.h>
+#include <openssl/x509v3.h>
+#endif
 }
 
 #include "nsscommon.h"
@@ -60,7 +65,11 @@ server_cert_nickname ()
 
 string
 add_cert_db_prefix (const string &db_path) {
-#if (NSS_VMAJOR > 3) || (NSS_VMAJOR == 3 && NSS_VMINOR >= 12)
+#if (NSS_VMAJOR > 3) || (NSS_VMAJOR == 3 && NSS_VMINOR >= 55)
+  // https://wiki.mozilla.org/NSS_Shared_DB
+  if (db_path.find (':') == string::npos)
+    return string("sql:") + db_path;
+#elif (NSS_VMAJOR > 3) || (NSS_VMAJOR == 3 && NSS_VMINOR >= 12)
   // Use the dbm prefix, if a prefix is not already specified,
   // since we're using the old database format.
   if (db_path.find (':') == string::npos)
@@ -105,7 +114,7 @@ nsscommon_error (const string &msg, int logit)
 static ofstream logfile;
 
 void
-start_log (const char *arg)
+start_log (const char *arg, bool redirect_clog)
 {
   if (logfile.is_open ())
     logfile.close ();
@@ -113,6 +122,8 @@ start_log (const char *arg)
   logfile.open (arg, ios_base::app);
   if (! logfile.good ())
     nsscommon_error (_F("Could not open log file %s", arg));
+  else if (redirect_clog)
+    clog.rdbuf(logfile.rdbuf());
 }
 
 bool
@@ -153,25 +164,46 @@ nssError (void)
   // See if PR_GetError can tell us what the error is.
   PRErrorCode errorNumber = PR_GetError ();
 
-  // PR_ErrorToString always returns a valid string for errors in this range.
-  if (errorNumber >= PR_NSPR_ERROR_BASE && errorNumber <= PR_MAX_ERROR)
-    {
-      nsscommon_error (_F("(%d) %s", errorNumber, PR_ErrorToString (errorNumber, PR_LANGUAGE_EN)));
-      return;
-    }
+   // Try PR_ErrorToName and PR_ErrorToString; they are wise.
+   const char* errorName = PR_ErrorToName(errorNumber);
+   const char* errorText = PR_ErrorToString (errorNumber, PR_LANGUAGE_EN);
 
-  // PR_ErrorToString does not handle errors outside the range above, so we handle them ourselves.
-  const char *errorText;
-  switch (errorNumber) {
-  default: errorText = "Unknown error"; break;
-#define NSSYERROR(code,msg) case code: errorText = msg; break
-#include "stapsslerr.h"
-#undef NSSYERROR
-    }
+   if (errorName && errorText)
+     {
+       nsscommon_error (_F("(%d %s) %s", errorNumber, errorName, errorText));
+       return;
+     }
 
+  errorText = "Unknown error";
   nsscommon_error (_F("(%d) %s", errorNumber, errorText));
 }
 
+extern "C"
+NSSInitContext*
+nssInitContext (const char *db_path, int readWrite, int issueMessage)
+{
+  NSSInitContext *context;
+  PRUint32 flags = (readWrite ? NSS_INIT_READONLY : 0) | NSS_INIT_OPTIMIZESPACE;
+
+  string full_db_path = add_cert_db_prefix (db_path);
+  db_path = full_db_path.c_str();
+  // DEBUG
+  char *sd = getenv("SYSTEMTAP_DIR");
+  string nssil_tmp = "/tmp/nssinit.log";
+  std::ofstream nssil_out(nssil_tmp, std::ofstream::app);
+  nssil_out << "NI db=" << db_path << " env=" << (sd ? sd : "") << (readWrite ? " rw" : " r") << endl;
+  nssil_out.close();
+  // DEBUG
+
+  context = NSS_InitContext (db_path, "", "", SECMOD_DB, NULL, flags);
+  if (context == NULL && issueMessage)
+    {
+      nsscommon_error (_F("Error initializing NSS for %s", db_path));
+      nssError ();
+    }
+  return context;
+}
+
 extern "C"
 SECStatus
 nssInit (const char *db_path, int readWrite, int issueMessage)
@@ -179,6 +211,7 @@ nssInit (const char *db_path, int readWrite, int issueMessage)
   SECStatus secStatus;
   string full_db_path = add_cert_db_prefix (db_path);
   db_path = full_db_path.c_str();
+  
   if (readWrite)
     secStatus = NSS_InitReadWrite (db_path);
   else
@@ -193,7 +226,7 @@ nssInit (const char *db_path, int readWrite, int issueMessage)
 
 extern "C"
 void
-nssCleanup (const char *db_path)
+nssCleanup (const char *db_path, NSSInitContext *context)
 {
   // Make sure that NSS has been initialized. Some early versions of NSS do not check this
   // within NSS_Shutdown().
@@ -216,7 +249,13 @@ nssCleanup (const char *db_path)
 
   // Shutdown NSS and ensure that it went down successfully. This is because we can not
   // initialize NSS again if it does not.
-  if (NSS_Shutdown () != SECSuccess)
+
+  SECStatus secStatus;
+  if (context != NULL)
+    secStatus = NSS_ShutdownContext (context);
+  else
+    secStatus = NSS_Shutdown ();
+  if (secStatus != SECSuccess)
     {
       if (db_path)
 	{
@@ -349,6 +388,7 @@ clean_cert_db (const string &db_path)
 	  if (remove_file_or_dir (globbuf.gl_pathv[i]) != 0)
 	    nsscommon_error (_F("Could not remove %s", globbuf.gl_pathv[i]));
 	}
+      globfree(& globbuf);
     }
 
   // Now remove the directory itself.
@@ -439,7 +479,7 @@ generate_private_key (const string &db_path, PK11SlotInfo *slot, SECKEYPublicKey
 
   // Set up for RSA.
   PK11RSAGenParams rsaparams;
-  rsaparams.keySizeInBits = 1024;
+  rsaparams.keySizeInBits = 4096; /* 1024 too small; SEC_ERROR_CERT_SIGNATURE_ALGORITHM_DISABLED */
   rsaparams.pe = 0x010001;
   CK_MECHANISM_TYPE mechanism = CKM_RSA_PKCS_KEY_PAIR_GEN;
 
@@ -697,7 +737,11 @@ static SECStatus
 add_server_cert (const string &db_path, SECItem *certDER, PK11SlotInfo *slot)
 {
   // Decode the cert.
-  CERTCertificate *cert = CERT_DecodeCertFromPackage((char *)certDER->data, certDER->len);
+  CERTCertificate *cert = CERT_NewTempCertificate(CERT_GetDefaultCertDB (), certDER,
+      (char*)server_cert_nickname () /* nickname */,
+      PR_FALSE /* isPerm */,
+      PR_TRUE /* copyDER */);
+
   if (! cert)
     {
       nsscommon_error (_("Unable to decode certificate"));
@@ -751,7 +795,7 @@ done:
 }
 
 SECStatus
-add_client_cert (const string &inFileName, const string &db_path)
+add_client_cert (const string &inFileName, const string &db_path, db_init_types db_init_type)
 {
   FILE *inFile = fopen (inFileName.c_str (), "rb");
   if (! inFile)
@@ -791,32 +835,50 @@ add_client_cert (const string &inFileName, const string &db_path)
       return SECFailure;
     }
 
-  // See if the database already exists and can be initialized.
-  SECStatus secStatus = nssInit (db_path.c_str (), 1/*readwrite*/, 0/*issueMessage*/);
-  if (secStatus != SECSuccess)
+  NSSInitContext *context = NULL;
+  if (db_init_type != db_no_nssinit)
     {
-      // Try again with a fresh database.
-      if (clean_cert_db (db_path.c_str ()) != 0)
-	{
-	  // Message already issued.
-	  return SECFailure;
-	}
-
-      // Make sure the given path exists.
-      if (create_client_cert_db (db_path.c_str ()) != 0)
-	{
-	  nsscommon_error (_F("Could not create certificate database directory %s",
-			      db_path.c_str ()));
-	  return SECFailure;
-	}
-
-      // Initialize the new database.
-      secStatus = nssInit (db_path.c_str (), 1/*readwrite*/);
+      // The http client adds a cert by calling us via add_server_trust which
+      // first has already called nssInit; we don't want to call nssInit twice
+      // See if the database already exists and can be initialized.
+      SECStatus secStatus = SECFailure;
+      if (db_init_type == db_nssinitcontext)
+        {
+          context= nssInitContext (db_path.c_str (), 1/*readwrite*/, 0/*issueMessage*/);
+          if (context == NULL)
+            secStatus = SECFailure;
+        }
+      else
+        secStatus = nssInit (db_path.c_str (), 1/*readwrite*/);
       if (secStatus != SECSuccess)
-	{
-	  // Message already issued.
-	  return SECFailure;
-	}
+        {
+          // Try again with a fresh database.
+          if (clean_cert_db (db_path.c_str ()) != 0)
+            {
+              // Message already issued.
+              return SECFailure;
+            }
+
+          // Make sure the given path exists.
+          if (create_client_cert_db (db_path.c_str ()) != 0)
+            {
+              nsscommon_error (_F("Could not create certificate database directory %s",
+                  db_path.c_str ()));
+              return SECFailure;
+            }
+
+          // Initialize the new database.
+          if (db_init_type == db_nssinitcontext)
+            {
+              context= nssInitContext (db_path.c_str (), 1/*readwrite*/);
+              if (context == NULL)
+                secStatus = SECFailure;
+            }
+          else
+              secStatus = nssInit (db_path.c_str (), 1/*readwrite*/);
+          if (secStatus != SECSuccess)
+            return SECFailure;
+        }
     }
 
   // Predeclare these to keep C++ happy about jumps to the label 'done'.
@@ -824,11 +886,16 @@ add_client_cert (const string &inFileName, const string &db_path)
   CERTCertDBHandle *handle = 0;
   CERTCertTrust *trust = 0;
   PK11SlotInfo *slot = 0;
+  SECStatus secStatus;
 
   // Add the cert to the database
   // Decode the cert.
   secStatus = SECFailure;
-  cert = CERT_DecodeCertFromPackage ((char *)certDER.data, certDER.len);
+  cert = CERT_NewTempCertificate(CERT_GetDefaultCertDB (), &certDER,
+      (char*)server_cert_nickname () /* nickname */,
+      PR_FALSE /* isPerm */,
+      PR_TRUE /* copyDER */);
+
   if (! cert)
     {
       nsscommon_error (_("Unable to decode certificate"));
@@ -855,6 +922,10 @@ add_client_cert (const string &inFileName, const string &db_path)
       goto done;
     }
   
+  if (PK11_NeedUserInit(slot)) {
+    PK11_InitPin(slot, (char*)NULL, "");
+  }
+  
   // Make it a trusted SSL peer.
   trust = (CERTCertTrust *)PORT_ZAlloc (sizeof (CERTCertTrust));
   if (! trust)
@@ -890,7 +961,8 @@ add_client_cert (const string &inFileName, const string &db_path)
     CERT_DestroyCertificate (cert);
   if (certDER.data)
     PORT_Free (certDER.data);
-  nssCleanup (db_path.c_str ());
+  if (db_init_type != db_no_nssinit)
+    nssCleanup (db_path.c_str (), context);
 
   // Make sure that the cert database files are read/write by the owner and
   // readable by all.
@@ -910,6 +982,7 @@ add_client_cert (const string &inFileName, const string &db_path)
 	  if (chmod (globbuf.gl_pathv[i], mode) != 0)
 	    nsscommon_error (_F("Could set file permissions for %s", globbuf.gl_pathv[i]));
 	}
+      globfree(&globbuf);
     }
   
   return secStatus;
@@ -957,6 +1030,7 @@ gen_cert_db (const string &db_path, const string &extraDnsNames, bool use_passwo
   int rc;
   string outFileName;
   FILE *outFile = 0;
+  secStatus = SECFailure;
 
   // We need the internal slot for this database.
   PK11SlotInfo *slot = PK11_GetInternalKeySlot ();
@@ -1080,7 +1154,7 @@ gen_cert_db (const string &db_path, const string &extraDnsNames, bool use_passwo
     CERT_DestroyCertificate (cert); // Also destroys certDER.
 
  done:
-  nssCleanup (db_path.c_str ());
+  nssCleanup (db_path.c_str (), NULL);
   return secStatus != SECSuccess;
 }
 
@@ -1161,9 +1235,58 @@ cert_is_valid (CERTCertificate *cert)
   return secStatus == SECSuccess;
 }
 
-static bool
-cert_db_is_valid (const string &db_path, const string &nss_cert_name)
+
+bool
+get_host_name (CERTCertificate *c, string &host_name)
 {
+  // Get the host name. It is in the alt-name extension of the
+  // certificate.
+
+  SECStatus secStatus;
+  PRArenaPool *tmpArena = NULL;
+  // A memory pool to work in
+  tmpArena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
+  if (! tmpArena)
+    {
+      clog << _("Out of memory:");
+      return false;
+    }
+
+  SECItem subAltName;
+  subAltName.data = NULL;
+  secStatus = CERT_FindCertExtension (c,
+				      SEC_OID_X509_SUBJECT_ALT_NAME,
+				      & subAltName);
+  if (secStatus != SECSuccess || ! subAltName.data)
+    {
+      clog << _("Unable to find alt name extension on server certificate: ") << endl;
+      PORT_FreeArena (tmpArena, PR_FALSE);
+      return false;
+    }
+
+  // Decode the extension.
+  CERTGeneralName *nameList = CERT_DecodeAltNameExtension (tmpArena, & subAltName);
+  SECITEM_FreeItem(& subAltName, PR_FALSE);
+  if (! nameList)
+    {
+      clog << _("Unable to decode alt name extension on server certificate: ") << endl;
+      return false;
+    }
+
+  // We're interested in the first alternate name.
+  assert (nameList->type == certDNSName);
+  host_name = string ((const char *)nameList->name.other.data,
+		      nameList->name.other.len);
+  PORT_FreeArena (tmpArena, PR_FALSE);
+  return true;
+}
+
+static bool
+cert_db_is_valid (const string &db_path, const string &nss_cert_name, 
+		  CERTCertificate *this_cert)
+{
+  NSSInitContext *context;
+
   // Make sure the given path exists.
   if (! file_exists (db_path))
     {
@@ -1179,8 +1302,8 @@ cert_db_is_valid (const string &db_path, const string &nss_cert_name)
     }
 
   // Initialize the NSS libraries -- readonly
-  SECStatus secStatus = nssInit (db_path.c_str ());
-  if (secStatus != SECSuccess)
+  context = nssInitContext (db_path.c_str ());
+  if (context == NULL)
     {
       // Message already issued.
       return false;
@@ -1217,6 +1340,8 @@ cert_db_is_valid (const string &db_path, const string &nss_cert_name)
 	  // The cert is valid. One valid cert is enough.
 	  log (_("Certificate is valid"));
 	  valid_p = true;
+	  if (this_cert)
+	    *this_cert = *c;
 	  break;
 	}
 
@@ -1226,16 +1351,186 @@ cert_db_is_valid (const string &db_path, const string &nss_cert_name)
   CERT_DestroyCertList (certs);
 
  done:
-  nssCleanup (db_path.c_str ());
+  nssCleanup (db_path.c_str (), context);
   return valid_p;
 }
 
+
+#ifdef HAVE_HTTP_SUPPORT
+/*
+ * Similar to cert_db_is_valid; additionally it checks host_name and does no logging
+ */
+static bool
+get_pem_cert_is_valid (const string &db_path, const string &nss_cert_name,
+                       const string &host_name, CERTCertificate *this_cert)
+{
+  NSSInitContext *context;
+  bool nss_already_init_p = false;
+  bool found_match = false;
+
+  if (! file_exists (db_path))
+    return false;
+
+  if (file_exists (db_path + "/pw"))
+    return false;
+
+  if (NSS_IsInitialized ())
+    nss_already_init_p = true;
+
+  context = nssInitContext (db_path.c_str (), 0 /* readWrite */, 0 /* issueMessage */);
+  if (context == NULL)
+    return false;
+
+  CERTCertList *certs = get_cert_list_from_db (nss_cert_name);
+  if (! certs)
+    {
+      nssCleanup (db_path.c_str (), context);
+      return false;
+    }
+
+  for (CERTCertListNode *node = CERT_LIST_HEAD (certs);
+       ! CERT_LIST_END (node, certs);
+       node = CERT_LIST_NEXT (node))
+    {
+      CERTCertificate *c = node->cert;
+      // An http client searches for the corresponding server's certificate
+      if (host_name.length() > 0)
+	{
+	  string cert_host_name;
+	  if (get_host_name (c, cert_host_name) == false)
+	      continue;
+
+	  if (cert_host_name != host_name)
+	    {
+	      size_t dot;
+	      if ((dot = host_name.find ('.')) == string::npos
+	          || cert_host_name != host_name.substr(0,dot))
+	        continue;
+	    }
+	}
+
+      // Now ask NSS to check the validity.
+      if (cert_is_valid (c))
+	{
+	  if (this_cert)
+	    *this_cert = *c;
+	  found_match = true;
+	  break;
+	}
+    }
+  CERT_DestroyCertList (certs);
+
+  // NSS shutdown is global and is forceful
+  if (!nss_already_init_p)
+    nssCleanup (db_path.c_str (), context);
+  return found_match;
+}
+
+
+bool
+cvt_nss_to_pem (CERTCertificate *c, string &cert_pem)
+{
+  BIO *bio;
+  X509 *certificate_509;
+
+  bio = BIO_new(BIO_s_mem());
+  certificate_509 = X509_new();
+  // Load the nss certificate into the openssl BIO
+  int count = BIO_write (bio, (const void*)c->derCert.data, c->derCert.len);
+  if (count == 0)
+    return false;
+  // Parse the BIO into an X509
+  certificate_509 = d2i_X509_bio(bio, &certificate_509);
+  // Convert the X509 to PEM form
+  int rc = PEM_write_bio_X509(bio, certificate_509);
+  if (rc != 1)
+    return false;
+  BUF_MEM *mem = NULL;
+  // Load the buffer from the PEM form
+  BIO_get_mem_ptr(bio, &mem);
+  cert_pem = string(mem->data, mem->length);
+  BIO_free(bio);
+  X509_free(certificate_509);
+  return true;
+}
+
+
+bool
+get_pem_cert (const string &db_path, const string &nss_cert_name, const string &host,
+    string &cert)
+{
+  CERTCertificate cc;
+  CERTCertificate *c = &cc;
+  // Do we have an nss certificate in the nss cert db for server host?
+  if (get_pem_cert_is_valid (db_path, nss_cert_name, host, c))
+    {
+      // If we do, then convert to PEM form
+      if (cvt_nss_to_pem (c, cert))
+        return true;
+    }
+  return false;
+}
+
+bool
+have_san_match (string & hostname, string & pem_cert)
+{
+  bool have_match = false;
+  STACK_OF (GENERAL_NAME) * san_names = NULL;
+
+  BIO *bio = BIO_new(BIO_s_mem());
+  BIO_puts(bio, pem_cert.c_str());
+  X509 *server_cert = PEM_read_bio_X509(bio, NULL, NULL, NULL);
+
+  // Extract the source alternate names from the certificate
+  san_names =
+    (stack_st_GENERAL_NAME *) X509_get_ext_d2i ((X509 *) server_cert,
+                                                NID_subject_alt_name, NULL,
+                                                NULL);
+  if (san_names == NULL)
+    {
+      return false;
+    }
+
+  for (int i = 0; i < sk_GENERAL_NAME_num (san_names); i++)
+    {
+      const GENERAL_NAME *current_name = sk_GENERAL_NAME_value (san_names, i);
+
+      if (current_name->type == GEN_DNS)
+        {
+          const int scheme_len = 8; // https://
+          string dns_name =
+#if OPENSSL_VERSION_NUMBER<0x10100000L
+              string ((char *) ASN1_STRING_data (current_name->d.dNSName));
+#else
+	      string ((char *) ASN1_STRING_get0_data (current_name->d.dNSName));
+#endif
+
+          if ((size_t) ASN1_STRING_length (current_name->d.dNSName) ==
+              dns_name.length ())
+            {
+              if (hostname.compare(scheme_len,dns_name.length(), dns_name))
+                {
+                  have_match = true;
+                  break;
+                }
+            }
+        }
+    }
+  sk_GENERAL_NAME_pop_free (san_names, GENERAL_NAME_free);
+  BIO_free(bio);
+  X509_free(server_cert);
+
+  return have_match;
+}
+#endif
+
+
 // Ensure that our certificate exists and is valid. Generate a new one if not.
 int
 check_cert (const string &db_path, const string &nss_cert_name, bool use_db_password)
 {
   // Generate a new cert database if the current one does not exist or is not valid.
-  if (! cert_db_is_valid (db_path, nss_cert_name))
+  if (! cert_db_is_valid (db_path, nss_cert_name, NULL))
     {
       if (gen_cert_db (db_path, "", use_db_password) != 0)
 	{
