@@ -315,10 +315,10 @@ stap_strfloctime(char *buf, size_t max, const char *fmt, time_t t)
 	if (buf == NULL || fmt == NULL || max <= 1)
 		return -EINVAL;
 	localtime_r(&t, &tm);
-        /* NB: this following invocation is the reason for stapdyn's
-           being built with -Wno-format-nonliteral.  strftime parsing
-           does not have security implications AFAIK, but gcc still
-           wants to check them. */
+	/* NB: this following invocation means that stapdyn modules can't be
+	   checked with -Wformat-nonliteral.  See compile_dyninst()
+	   buildrun.cxx for the flags chosen.  strftime parsing does not have
+	   security implications AFAIK, but gcc still wants to check them.  */
 	ret = strftime(buf, max, fmt, &tm);
 	if (ret == 0)
 		return -EINVAL;
