@@ -136,17 +136,18 @@ static struct _stp_module *_stp_kmod_sec_lookup(unsigned long addr,
 static struct _stp_module *_stp_umod_lookup(unsigned long addr,
 					    struct task_struct *task,
 					    const char **name,
+					    unsigned long *offset,
 					    unsigned long *vm_start,
 					    unsigned long *vm_end)
 {
   void *user = NULL;
 #ifdef CONFIG_COMPAT
         /* Handle 32bit signed values in 64bit longs, chop off top bits. */
-        if (test_tsk_thread_flag(task, TIF_32BIT))
+  if (_stp_is_compat_task2(task))
           addr &= ((compat_ulong_t) ~0);
 #endif
   if (stap_find_vma_map_info(task->group_leader, addr,
-			     vm_start, vm_end, name, &user) == 0)
+			     vm_start, vm_end, offset, name, &user) == 0)
     if (user != NULL)
       {
 	struct _stp_module *m = (struct _stp_module *)user;
@@ -175,22 +176,23 @@ static const char *_stp_kallsyms_lookup(unsigned long addr,
 
 	if (task)
 	  {
+	    unsigned long sect_offset = 0;
 	    unsigned long vm_start = 0;
 	    unsigned long vm_end = 0;
 #ifdef CONFIG_COMPAT
         /* Handle 32bit signed values in 64bit longs, chop off top bits.
            _stp_umod_lookup does the same, but we need it here for the
            binary search on addr below. */
-        if (test_tsk_thread_flag(task, TIF_32BIT))
-          addr &= ((compat_ulong_t) ~0);
+            if (_stp_is_compat_task2(task))
+                    addr &= ((compat_ulong_t) ~0);
 #endif
-	    m = _stp_umod_lookup(addr, task, modname, &vm_start, &vm_end);
+	    m = _stp_umod_lookup(addr, task, modname, &sect_offset, &vm_start, &vm_end);
 	    if (m)
 	      {
 		sec = &m->sections[0];
 		/* XXX .absolute sections really shouldn't be here... */
 		if (strcmp(".dynamic", m->sections[0].name) == 0)
-		  rel_addr = addr - vm_start;
+		  rel_addr = addr - vm_start + sect_offset;
 		else
 		  rel_addr = addr;
 	      }
@@ -353,14 +355,28 @@ static void _stp_filename_lookup(struct _stp_module *mod, char ** filename,
     }
 }
 
+// At this point the compiler needs to know the context struct.  The context
+// struct is emitted by s.up->emit_common_header () within translate.cxx way
+// after this file. To solve this problem, keep only the function declaration
+// here, and move the actual function definition out to a separate sym2.c,
+// which is included once the context struct is known.
+static void _stp_filename_lookup_5(struct _stp_module *mod, char ** filename,
+                                   uint8_t *dirsecp, uint8_t *enddirsecp,
+                                   unsigned int length,
+                                   unsigned fileidx, int user, int compat_task,
+                                   struct context *c);
+
 #endif /* STP_NEED_LINE_DATA */
 
-unsigned long _stp_linenumber_lookup(unsigned long addr, struct task_struct *task, char ** filename, int need_filename)
+unsigned long _stp_linenumber_lookup(unsigned long addr, struct task_struct *task,
+                                     char ** filename, int need_filename,
+                                     struct context *c)
 {
   struct _stp_module *m;
   struct _stp_section *sec;
   const char *modname = NULL;
   uint8_t *linep, *enddatap;
+  uint8_t *str_linep, *str_enddatap;
   int compat_task = _stp_is_compat_task();
   int user = (task ? 1 : 0);
 
@@ -372,14 +388,12 @@ unsigned long _stp_linenumber_lookup(unsigned long addr, struct task_struct *tas
 
   if (task)
     {
-	    unsigned long vm_start = 0;
-	    unsigned long vm_end = 0;
 #ifdef CONFIG_COMPAT
       /* Handle 32bit signed values in 64bit longs, chop off top bits. */
-      if (test_tsk_thread_flag(task, TIF_32BIT))
-        addr &= ((compat_ulong_t) ~0);
+            if (_stp_is_compat_task2(task))
+                    addr &= ((compat_ulong_t) ~0);
 #endif
-	    m = _stp_umod_lookup(addr, task, &modname, &vm_start, &vm_end);
+	    m = _stp_umod_lookup(addr, task, &modname, NULL, NULL, NULL);
     }
   else
     m = _stp_kmod_sec_lookup(addr, &sec);
@@ -405,7 +419,7 @@ unsigned long _stp_linenumber_lookup(unsigned long addr, struct task_struct *tas
       addr = addr - offset;
     }
 
-
+  // The .debug_line section
   linep = m->debug_line;
   enddatap = m->debug_line + m->debug_line_len;
 
@@ -442,6 +456,16 @@ unsigned long _stp_linenumber_lookup(unsigned long addr, struct task_struct *tas
 
       version = read_pointer ((const uint8_t **) &linep, endunitp, DW_EH_PE_data2, user, compat_task);
 
+      // Need to skip over DWARF 5's address_size and segment_selector_size right
+      // to hdr_length (analogy to what happens in pass2's dump_line_tables_check()
+      // PR29984
+      if (version >= 5)
+        {
+          if (unlikely (linep + 2 > enddatap))
+            return 0;
+          linep += 2;
+        }
+
       if (length == 4)
         hdr_length = (uint64_t) read_pointer ((const uint8_t **) &linep, endunitp, DW_EH_PE_data4, user, compat_task);
       else
@@ -613,8 +637,15 @@ unsigned long _stp_linenumber_lookup(unsigned long addr, struct task_struct *tas
             if (row_end_sequence == 0 && row_addr <= addr && addr < curr_addr)
               {
                 if (need_filename)
-                  _stp_filename_lookup(m, filename, dirsecp, endhdrp,
-                                       row_file_idx, user, compat_task);
+                  {
+                    if (version > 4)
+                      _stp_filename_lookup_5(m, filename, dirsecp, endhdrp,
+                                             length, row_file_idx, user,
+                                             compat_task, c);
+                    else
+                      _stp_filename_lookup(m, filename, dirsecp, endhdrp,
+                                           row_file_idx, user, compat_task);
+                  }
                 return row_linenum;
               }
 
@@ -662,45 +693,59 @@ static int _stp_build_id_check (struct _stp_module *m,
 
   memset(hexstring_practice, '\0', max_buildid_hexstring);
   for (i=0, j=0; j<buildid_len; j++) {
+#ifdef STAPCONF_SET_FS
     /* Use set_fs / get_user to access conceivably invalid addresses.
      * If loc2c-runtime.h were more easily usable, a deref() loop
      * could do it too. */
     mm_segment_t oldfs = get_fs();
+#endif
     int rc;
     unsigned char practice = 0;
 
 #ifdef STAPCONF_PROBE_KERNEL
     if (!tsk) {
+#ifdef STAPCONF_SET_FS
       set_fs(KERNEL_DS);
+#endif
       rc = probe_kernel_read(&practice, (void*)(notes_addr + j), 1);
     }
     else
 #endif
     {
+#ifdef STAPCONF_SET_FS
       set_fs (tsk ? USER_DS : KERNEL_DS);
+#endif
 
       /*
-       * Why check CONFIG_UTRACE here? If we're using real in-kernel
-       * utrace, we can always just call get_user() (since we're
-       * either reading kernel memory or tsk == current).
-       *
        * Since we're only reading here, we can call
        * __access_process_vm_noflush(), which only calls things that
        * are exported.
+       *
+       * PR26811: Kernel 5.10+ after set_fs() removal no longer supports
+       * reading kernel addresses with get_user.
        */
-#ifdef CONFIG_UTRACE
-      rc = get_user(practice, ((unsigned char*)(void*)(notes_addr + j)));
-#else
+#if defined(STAPCONF_SET_FS)
       if (!tsk || tsk == current) {
 	rc = get_user(practice, ((unsigned char*)(void*)(notes_addr + j)));
       }
+#else
+      if (!tsk) {
+        /* XXX Just reading kernel memory should be sufficient
+           e.g. per Linux kernel commit ee6e00c8. */
+        practice = *((unsigned char*)(void*)(notes_addr + j));
+        rc = 0;
+      } else if (tsk == current) {
+	rc = get_user(practice, ((unsigned char*)(void*)(notes_addr + j)));
+      }
+#endif
       else {
 	rc = (__access_process_vm_noflush(tsk, (notes_addr + j), &practice,
 					  1, 0) != 1);
       }
-#endif
     }
+#ifdef STAPCONF_SET_FS
     set_fs(oldfs);
+#endif
 
     if (rc == 0) { // got actual data byte
             hexstring_practice[i++] = hexnibble[practice >> 4];
@@ -708,14 +753,18 @@ static int _stp_build_id_check (struct _stp_module *m,
     }
   }
 
+  dbug_sym(1, "build-id validation [%s] %s vs. notes@=%lx %s\n", m->name, hexstring_theory,
+           (unsigned long) notes_addr, hexstring_practice);
+  
   // have two strings, will travel
   if (strcmp (hexstring_practice, hexstring_theory)) {
 	  // NB: It is normal for different binaries with the same file path
 	  // coexist in the same system via chroot or namespaces, therefore
 	  // we make sure below is really a warning.
-          _stp_warn ("Build-id mismatch [man warning::buildid]: \"%s\" address "
+          _stp_warn ("Build-id mismatch [man warning::buildid]: \"%s\" pid %ld address "
 		     "%#lx, expected %s actual %s\n",
-                     m->path, notes_addr, hexstring_theory, hexstring_practice);
+                     m->path, (long) (tsk ? tsk->tgid : 0),
+                     notes_addr, hexstring_theory, hexstring_practice);
       return 1;
   }
   
@@ -827,7 +876,6 @@ static int _stp_usermodule_check(struct task_struct *tsk, const char *path_name,
   struct _stp_module *m = NULL;
   unsigned long notes_addr;
   unsigned i, j;
-  unsigned char practice_id_bits[MAXSTRINGLEN];
   unsigned long vm_end = 0;
 
 #ifdef STP_NO_BUILDID_CHECK
@@ -869,7 +917,7 @@ static int _stp_usermodule_check(struct task_struct *tsk, const char *path_name,
  * @note Symbolic lookups should not normally be done within
  * a probe because it is too time-consuming. Use at module exit time. */
 static int _stp_snprint_addr(char *str, size_t len, unsigned long address,
-			     int flags, struct task_struct *task)
+			     int flags, struct task_struct *task, struct context *c)
 {
   const char *modname = NULL;
   const char *name = NULL;
@@ -901,7 +949,7 @@ static int _stp_snprint_addr(char *str, size_t len, unsigned long address,
 
   if ((flags & _STP_SYM_LINENUMBER) || (flags & _STP_SYM_FILENAME)) {
       linenumber = _stp_linenumber_lookup (address, task, &filename,
-                                           (int) (flags & _STP_SYM_FILENAME));
+                                           (int) (flags & _STP_SYM_FILENAME), c);
   }
 
   switch (flags & ~_STP_SYM_INEXACT) {
@@ -1048,9 +1096,10 @@ static int _stp_snprint_addr(char *str, size_t len, unsigned long address,
 }
 
 static void _stp_print_addr(unsigned long address, int flags,
-			    struct task_struct *task)
+			    struct task_struct *task,
+                            struct context *c)
 {
-  _stp_snprint_addr(NULL, 0, address, flags, task);
+  _stp_snprint_addr(NULL, 0, address, flags, task, c);
 }
 
 /** @} */
@@ -1113,7 +1162,8 @@ unsigned long kallsyms_lookup_name (const char *name)
         /* First, try to use a kallsyms_lookup_name address passed to us
            through the relocation mechanism. */
         if (_stp_kallsyms_lookup_name != NULL)
-                return (* (kallsyms_lookup_name_fn)_stp_kallsyms_lookup_name)(name);
+	  return ibt_wrapper(unsigned long,
+			     (* (kallsyms_lookup_name_fn)_stp_kallsyms_lookup_name)(name));
 
         /* NB: PR14804: definitely don't use _stp_error here.  It's called
            too early for the actual message buffer goo to be allocated. */
@@ -1132,14 +1182,21 @@ unsigned long kallsyms_lookup_name (const char *name)
 typedef typeof(&kallsyms_on_each_symbol) kallsyms_on_each_symbol_fn;
 
 // XXX Will be linked in place of the kernel's kallsyms_on_each_symbol:
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(6,4,0)
+int kallsyms_on_each_symbol(int (*fn)(void *, const char *,
+				      unsigned long),
+                            void *data)
+#else
 int kallsyms_on_each_symbol(int (*fn)(void *, const char *, struct module *,
 				      unsigned long),
                             void *data)
+#endif
 {
         /* First, try to use a kallsyms_lookup_name address passed to us
            through the relocation mechanism. */
         if (_stp_kallsyms_on_each_symbol != NULL)
-                return (* (kallsyms_on_each_symbol_fn)_stp_kallsyms_on_each_symbol)(fn, data);
+	  return ibt_wrapper(int,
+			     (* (kallsyms_on_each_symbol_fn)_stp_kallsyms_on_each_symbol)(fn, data));
 
         /* Next, give up and signal a BUG. We should have detected
            that this function is not available and used a different
@@ -1147,6 +1204,48 @@ int kallsyms_on_each_symbol(int (*fn)(void *, const char *, struct module *,
         _stp_error("BUG: attempting to use unavailable kallsyms_on_each_symbol!!\n");
         return 0;
 }
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5,12,0)
+typedef typeof(&module_kallsyms_on_each_symbol) module_kallsyms_on_each_symbol_fn;
+
+// XXX Will be linked in place of the kernel's module_kallsyms_on_each_symbol:
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(6,4,0)
+int module_kallsyms_on_each_symbol(const char *modname,
+                                   int (*fn)(void *, const char *,
+				      unsigned long),
+                            void *data)
+#else
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(6,3,0)
+int module_kallsyms_on_each_symbol(const char *modname,
+				   int (*fn)(void *, const char *, struct module *,
+				      unsigned long),
+                            void *data)
+#else
+int module_kallsyms_on_each_symbol(int (*fn)(void *, const char *, struct module *,
+				      unsigned long),
+                            void *data)
+#endif
+#endif
+{
+        /* First, try to use a kallsyms_lookup_name address passed to us
+           through the relocation mechanism. */
+        if (_stp_module_kallsyms_on_each_symbol != NULL)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(6,3,0)
+		return ibt_wrapper(int,
+				   (* (module_kallsyms_on_each_symbol_fn)_stp_module_kallsyms_on_each_symbol)(modname, fn, data));
+#else
+		return ibt_wrapper(int,
+				   (* (module_kallsyms_on_each_symbol_fn)_stp_module_kallsyms_on_each_symbol)(fn, data));
+#endif
+
+        /* Next, give up and signal a BUG. We should have detected
+           that this function is not available and used a different
+           mechanism! */
+        _stp_error("BUG: attempting to use unavailable module_kallsyms_on_each_symbol!!\n");
+        return 0;
+}
+#endif
+
 #endif
 
 
