@@ -57,7 +57,7 @@ static ssize_t _stp_ctl_write_cmd(struct file *file, const char __user *buf, siz
 
 #if defined(DEBUG_TRANS) && (DEBUG_TRANS >= 2)
 	if (type < STP_MAX_CMD)
-		dbug_trans2("Got %s. euid=%ld, len=%d\n", _stp_command_name[type],
+		dbug_trans2("Got %s. euid=%ld, len=%d\n", _stp_command_name[min(type,STP_MAX_CMD)] ?: "?",
 			    (long)euid, (int)count);
 #endif
 
@@ -211,7 +211,9 @@ out:
 
 #if defined(DEBUG_TRANS) && (DEBUG_TRANS >= 2)
 	if (type < STP_MAX_CMD)
-		dbug_trans2("Completed %s (rc=%d)\n", _stp_command_name[type], rc);
+		dbug_trans2("Completed %s (rc=%d)\n",
+                            _stp_command_name[min(type,STP_MAX_CMD)] ?: "?",
+                            rc);
 #endif
         return rc;
 }
