@@ -131,7 +131,7 @@ stapregex_compile (regexp *re, const std::string& match_snippet,
 
 /* Now follows the heart of the tagged-DFA algorithm. This is a basic
    implementation of the algorithm described in Ville Laurikari's
-   Masters thesis and summarized in the paper "NFAs with Tagged
+   Main thesis and summarized in the paper "NFAs with Tagged
    Transitions, their Conversion to Deterministic Automata and
    Application to Regular Expressions"
    (http://laurikari.net/ville/spire2000-tnfa.pdf).
@@ -1020,7 +1020,7 @@ span::emit_jump (translator_output *o, const dfa *d) const
 
   if (to->accepts)
     {
-      emit_final(o, d);
+      emit_final(o, d, false /*saw_nul*/);
       return;
     }
 
@@ -1033,7 +1033,7 @@ span::emit_jump (translator_output *o, const dfa *d) const
 /* Assuming the target DFA state of the span is a final state, emit code to
    cleanup tags and (if appropriate) exit with a final answer. */
 void
-span::emit_final (translator_output *o, const dfa *d) const
+span::emit_final (translator_output *o, const dfa *d, bool saw_nul) const
 {
   assert (to->accepts); // XXX: must guarantee correct usage of emit_final()
 
@@ -1087,6 +1087,8 @@ span::emit_final (translator_output *o, const dfa *d) const
       o->indent(-1);
       o->newline() << "}";
 
+      if (saw_nul)
+        o->newline () << "YYCURSOR--;"; /* PR30395: the next state should encounter a repeated NUL */
       o->newline () << "goto yystate" << to->label << ";";
     }
 }
@@ -1119,10 +1121,11 @@ state::emit (translator_output *o, const dfa *d) const
       if (it->lb == '\0')
         {
           o->newline() << "case " << c_char('\0') << ":";
-          it->emit_final(o, d);
+          it->emit_final(o, d, true /* saw_nul */);
         }
 
       // Emit labels to handle all the other elements of the span:
+      bool has_element = false;
       for (unsigned c = max((rchar) '\1', it->lb);
            c <= (unsigned) it->ub; c++) {
         if (c > 127)
@@ -1131,8 +1134,10 @@ state::emit (translator_output *o, const dfa *d) const
             continue; // XXX: not an ASCII char, needs special handling
           }
         o->newline() << "case " << c_char((rchar) c) << ":";
+        has_element = true;
       }
-      it->emit_jump(o, d);
+      if (has_element) /* avoid duplicate action on a span of only '\0' */
+          it->emit_jump(o, d);
 
       // TODOXXX 'default' option should handle the largest span
       // TODOXXX optimize by accepting before end of string whenever possible
