@@ -29,6 +29,7 @@
 #include <sys/uio.h>
 #include <glob.h>
 #include <time.h>
+#include <unistd.h>
 #include <sys/prctl.h>
 #include <sys/utsname.h>
 
@@ -242,7 +243,11 @@ static void remove_all_modules(void)
 
 static int remove_module(const char *name, int verb)
 {
-	int ret;
+	int i, ret;
+	enum {
+		MAX_EINTR_TRIES = 5
+	};
+
 	dbug(2, "%s\n", name);
 
 #ifdef PR_SET_NAME
@@ -271,12 +276,15 @@ static int remove_module(const char *name, int verb)
 
 	dbug(2, "removing module %s\n", name);
 	PROBE1(staprun, remove__module, name);
-	ret = delete_module (name, O_NONBLOCK);
+
+	for (i = 0; i < MAX_EINTR_TRIES; i++) {
+		ret = delete_module (name, O_NONBLOCK);
+		if (ret == 0 || (errno != EINTR && errno != EWOULDBLOCK))
+			break;
+		usleep(100 * i);
+	}
+
 	if (ret != 0) {
-                /* XXX: maybe we should just accept this, with a
-                   diagnostic, but without an error.  Might it be
-                   possible for the same module to be started up just
-                   as we're shutting down?  */
 		err("Couldn't remove module '%s': %s.\n", name, strerror(errno));
 		return 1;
 	}
@@ -438,6 +446,16 @@ int main(int argc, char **argv)
 {
 	int rc;
 
+	/* Force libc to make our stderr messages atomic by enabling line
+	   buffering since stderr is unbuffered by default. Without this, libc
+	   is at liberty to split a single stderr message into multiple writes
+	   to the fd while holding flockfile(stderr). POSIX only guarantees that
+	   a single write(2) is atomic; chaining several write(2) calls together
+	   won't be atomic, and we don't want libc to do that within a single
+	   *fprintf(stderr) call since it'll mangle messages printed across
+	   different processes (*not* threads). */
+	setlinebuf(stderr);
+
 	/* NB: Don't do the geteuid()!=0 check here, since we want to
 	   test command-line error-handling while running non-root. */
 	/* Get rid of a few standard environment variables (which */
@@ -622,6 +640,7 @@ int send_relocation_kernel ()
       int found_stext = 0;
       int found_kallsyms_lookup_name = 0;
       int found_kallsyms_on_each_symbol = 0;
+      int found_module_kallsyms_on_each_symbol = 0;
       int done_with_kallsyms = 0;
       char *line = NULL;
       size_t linesz = 0;
@@ -663,10 +682,20 @@ int send_relocation_kernel ()
 
                   found_kallsyms_on_each_symbol = 1;
                 }
+              else if (linesize - pos == sizeof "module_kallsyms_on_each_symbol"
+                       && !strcmp(line + pos, "module_kallsyms_on_each_symbol" "\n"))
+                {
+                  rc = send_a_relocation ("kernel", "module_kallsyms_on_each_symbol", address);
+		  if (rc != 0) // non fatal, follows perror()
+                    dbug(1, "Relocation was reloc module_kallsyms_on_each_symbol=%llx\n", address);
+
+                  found_module_kallsyms_on_each_symbol = 1;
+                }
             }
           done_with_kallsyms = found_stext
             && found_kallsyms_lookup_name
-            && found_kallsyms_on_each_symbol;
+            && found_kallsyms_on_each_symbol
+            && found_module_kallsyms_on_each_symbol;
         }
       free (line);
       fclose (kallsyms);
