@@ -1,5 +1,5 @@
 // tapset for timers
-// Copyright (C) 2005-2011 Red Hat Inc.
+// Copyright (C) 2005-2019 Red Hat Inc.
 // Copyright (C) 2005-2007 Intel Corporation.
 //
 // This file is part of systemtap, and is free software.  You can
@@ -47,6 +47,10 @@ struct timer_derived_probe: public derived_probe
 struct timer_derived_probe_group: public generic_dpg<timer_derived_probe>
 {
   void emit_interval (translator_output* o);
+  friend bool sort_for_bpf(systemtap_session& s,
+			   hrtimer_derived_probe_group *hr,
+                           timer_derived_probe_group *t,
+                           sort_for_bpf_probe_arg_vector &v);
 public:
   void emit_module_decls (systemtap_session& s);
   void emit_module_init (systemtap_session& s);
@@ -119,16 +123,20 @@ timer_derived_probe_group::emit_module_decls (systemtap_session& s)
   s.op->newline(-1) << "};";
   s.op->newline();
 
-  s.op->newline() << "static void enter_timer_probe (unsigned long val) {";
+  s.op->newline() << "static void enter_timer_probe (stp_timer_callback_parameter_t val) {";
+  s.op->newline() << "#if !defined(init_timer)";
+  s.op->newline(1) << "struct stap_timer_probe* stp = container_of(val, struct stap_timer_probe, timer_list);";
+  s.op->newline(-1) << "#else";
   s.op->newline(1) << "struct stap_timer_probe* stp = & stap_timer_probes [val];";
-  s.op->newline() << "if ((atomic_read (session_state()) == STAP_SESSION_STARTING) ||";
+  s.op->newline(-1) << "#endif";
+  s.op->newline(1) << "if ((atomic_read (session_state()) == STAP_SESSION_STARTING) ||";
   s.op->newline() << "    (atomic_read (session_state()) == STAP_SESSION_RUNNING))";
   s.op->newline(1) << "mod_timer (& stp->timer_list, jiffies + ";
   emit_interval (s.op);
   s.op->line() << ");";
   s.op->newline(-1) << "{";
   s.op->indent(1);
-  common_probe_entryfn_prologue (s, "STAP_SESSION_RUNNING", "stp->probe",
+  common_probe_entryfn_prologue (s, "STAP_SESSION_RUNNING", "", "stp->probe",
 				 "stp_probe_type_timer");
   s.op->newline() << "(*stp->probe->ph) (c);";
   common_probe_entryfn_epilogue (s, true, otf_safe_context(s));
@@ -145,9 +153,11 @@ timer_derived_probe_group::emit_module_init (systemtap_session& s)
   s.op->newline() << "for (i=0; i<" << probes.size() << "; i++) {";
   s.op->newline(1) << "struct stap_timer_probe* stp = & stap_timer_probes [i];";
   s.op->newline() << "probe_point = stp->probe->pp;";
-  s.op->newline() << "init_timer (& stp->timer_list);";
-  s.op->newline() << "stp->timer_list.function = & enter_timer_probe;";
+
+  s.op->newline() << "timer_setup (& stp->timer_list, enter_timer_probe, 0);";
+  s.op->newline() << "#if defined(init_timer)";
   s.op->newline() << "stp->timer_list.data = i;"; // NB: important!
+  s.op->newline() << "#endif";
   // copy timer renew calculations from above :-(
   s.op->newline() << "stp->timer_list.expires = jiffies + ";
   emit_interval (s.op);
@@ -217,6 +227,10 @@ struct hrtimer_derived_probe: public derived_probe
 
 struct hrtimer_derived_probe_group: public generic_dpg<hrtimer_derived_probe>
 {
+  friend bool sort_for_bpf(systemtap_session& s,
+			   hrtimer_derived_probe_group *hr,
+                           timer_derived_probe_group *t,
+                           sort_for_bpf_probe_arg_vector &v);
 public:
   void emit_module_decls (systemtap_session& s);
   void emit_module_init (systemtap_session& s);
@@ -279,7 +293,7 @@ hrtimer_derived_probe_group::emit_module_decls (systemtap_session& s)
 
       s.op->newline() << "{";
       s.op->indent(1);
-      common_probe_entryfn_prologue (s, "STAP_SESSION_RUNNING", "stp->probe",
+      common_probe_entryfn_prologue (s, "STAP_SESSION_RUNNING", "", "stp->probe",
 				     "stp_probe_type_hrtimer");
       s.op->newline() << "(*stp->probe->ph) (c);";
       common_probe_entryfn_epilogue (s, true, otf_safe_context(s));
@@ -301,7 +315,8 @@ hrtimer_derived_probe_group::emit_module_decls (systemtap_session& s)
 
       s.op->newline() << "{";
       s.op->indent(1);
-      common_probe_entryfn_prologue (s, "STAP_SESSION_RUNNING", "stp->probe",
+      common_probe_entryfn_prologue (s, "STAP_SESSION_RUNNING", "",
+                                     "stp->probe",
 				     "stp_probe_type_hrtimer");
       s.op->newline() << "(*stp->probe->ph) (c);";
       common_probe_entryfn_epilogue (s, true, otf_safe_context(s));
@@ -424,6 +439,10 @@ public:
   void emit_module_decls (systemtap_session& s);
   void emit_module_init (systemtap_session& s);
   void emit_module_exit (systemtap_session& s);
+
+  friend void warn_for_bpf(systemtap_session& s,
+                           profile_derived_probe_group *dpg,
+                           const std::string& kind);
 };
 
 
@@ -469,7 +488,7 @@ profile_derived_probe_group::emit_module_decls (systemtap_session& s)
   s.op->newline() << "static void enter_all_profile_probes (struct pt_regs *regs) {";
   s.op->newline(1) << "const struct stap_probe * probe = "
                    << common_probe_init (probes[0]) << ";";
-  common_probe_entryfn_prologue (s, "STAP_SESSION_RUNNING", "probe",
+  common_probe_entryfn_prologue (s, "STAP_SESSION_RUNNING", "", "probe",
 				 "stp_probe_type_profile_timer");
   // Timer interrupts save all registers, so if the interrupt happened
   // in user space we can rely on it being the full user pt_regs.
@@ -550,6 +569,20 @@ profile_derived_probe_group::emit_module_exit (systemtap_session& s)
 }
 
 
+// PR26234: Not supported by stapbpf.
+void
+warn_for_bpf(systemtap_session& s,
+             profile_derived_probe_group *dpg,
+             const std::string& kind)
+{
+  for (unsigned i=0; i<dpg->probes.size(); i++)
+    {
+      s.print_warning(_F("%s will be ignored by bpf backend",
+                         kind.c_str()),
+                      dpg->probes[i]->tok);
+    }
+}
+
 
 // ------------------------------------------------------------------------
 // unified probe builder for timer probes
@@ -564,6 +597,8 @@ struct timer_builder: public derived_probe_builder
                        vector<derived_probe *> & finished_results);
 
     static void register_patterns(systemtap_session& s);
+
+    virtual string name() { return "timer builder"; }
 };
 
 void
@@ -751,6 +786,37 @@ register_tapset_timers(systemtap_session& s)
   }
 }
 
+bool
+sort_for_bpf(systemtap_session& s __attribute__ ((unused)),
+	     hrtimer_derived_probe_group *hr, timer_derived_probe_group *t,
+             sort_for_bpf_probe_arg_vector &v)
+{
+  if (hr)
+    for (auto i = hr->probes.begin(); i != hr->probes.end(); ++i)
+      {
+        hrtimer_derived_probe *p = *i;
+        std::stringstream o;
+
+        o << "timer/nsec/" << p->interval;
+        v.push_back(std::pair<derived_probe *, std::string>(p, o.str()));
+      }
+
+  if (t)
+    for (auto i = t->probes.begin(); i != t->probes.end(); ++i)
+      {
+        timer_derived_probe *p = *i;
+        std::stringstream o;
+
+        if (p->time_is_msecs)
+          o << "timer/nsec/" << (p->interval * 1e6);
+        else
+          o << "timer/jiff/" << p->interval;
+
+        v.push_back(std::pair<derived_probe *, std::string>(p, o.str()));
+      }
+
+  return v.empty();
+}
 
 
 /* vim: set sw=2 ts=8 cino=>4,n-2,{2,^-2,t0,(0,u0,w1,M1 : */
