@@ -1,4 +1,5 @@
 #include "sys/sdt.h"
+#include <unistd.h>
 
 void sleeper () {
   sleep(1);
@@ -10,5 +11,7 @@ int main () {
         marker_here:
             STAP_PROBE(tmp_test_file, while_start);
     }
+    if (0) // suppress gcc warning about unused label
+      goto marker_here;
     return 0;
 }
